<?php
/*
Plugin Name: Local Syndication
Plugin URI: https://www.willmaster.com/software/WPplugins/
Description: Insert content of any page/file/script at any URL on the Internet into any WordPress post or page.
Version: 1.5a
Date: 8 December 2018
Author: Will Bontrager Software, LLC <will@willmaster.com>
Author URI: https://www.willmaster.com/contact.php
*/

/*
	Copyright 2011, 2013, 2018 Will Bontrager Software LLC (email: will@willmaster.com)

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License, version 2, as 
	published by the Free Software Foundation. A copy of the license is at
	https://www.gnu.org/licenses/old-licenses/gpl-2.0.html

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
*/

/*
Note: This plugin requires WordPress version 3.0 or higher.

Tested up to WP version 5.0

Information about the Local Syndication plugin can be found here:
https://www.willmaster.com/software/WPplugins/go/lshome_lsplugin

Instructions for using the Local Syndication plugin can be found here:
https://www.willmaster.com/software/WPplugins/go/lsinstructions_lsplugin
*/

if( ! function_exists('will_bontrager_local_syndication') )
{
	function will_bontrager_local_syndication($atts,$content=null,$code="")
	{
		$url = $atts['url'];
		if( strpos($url,':') === false and strpos($url,'/') !== 0 ) { $url = "/$url"; }
		if( strpos($url,'/') === 0 and strpos($url,'//') === false ) { $url = will_bontrager_local_syndication_make_httphttps($url) . '://' . $_SERVER['HTTP_HOST'] . $url; }
		$pre_result = $post_result = '';
		if( ! empty($atts['iframe']) )
		{
			foreach(array('url','iframe','js','script','plain','src','wrap_long_lines') as $key)
			{
			    if(isset($atts[$key])) { unset($atts[$key]); }
			}
			$s = "<iframe src='$url'";
			foreach( $atts as $k => $v ) { $s .= " $k='$v'"; }
			$s .= '>';
			$post_result = '</iframe>';
			if( $js )
			{
				$pre_result .= will_bontrager_local_syndication_js_line($s);
				$post_result = will_bontrager_local_syndication_js_line($post_result).'</script>';
			}
			else { $pre_result .= $s; }
			return "$pre_result$post_result";
		}
		$response = wp_remote_get($url);
		if( is_wp_error($response) )
		{
			$error_string = $response->get_error_message();
			return "WordPress error: $error_string";
		}
		if( empty($response['body']) and empty($response['response']['code']) )
		{
			return "Unable to obtain content at $url";
		}
		if( $response['response']['code'] != 200 )
		{
			return 'Unable to obtain content at<br>' . 
				"$url<br>" .
				$response['response']['code'] . ' - ' . $response['response']['message'];
		}
		$response_content = $response['body'];
		$js = empty($atts['js']) ? false : true;
		$isjs = empty($atts['script']) ? false : true;
		if( $js and $isjs ) { $js = false; }
		if( $js ) { $pre_result = '<script type="text/javascript">'; }
		elseif( $isjs )
		{
			if( ! preg_match('/^[^<]*<script/is',$response_content) )
			{
				$pre_result = '<script type="text/javascript">';
    			$post_result = '</script>';
			}
		}
		if( ! empty($atts['plain']) )
		{
		    if( ! empty($atts['showtags']) ) { $response_content = htmlspecialchars($response_content); }
			$s = empty($atts['wrap_long_lines']) ? '<pre style="white-space:pre; ' : '<div style="white-space:pre-wrap; ';
			if( isset($atts['style']) ) { $s .= $atts['style']; }
			$s .= '"';
			if( isset($atts['class']) ) { $s .= 'class="' . $atts['class'] . '" '; }
			$s .= '>';
			$post_result = '</div>';
			if( $js )
			{
				$pre_result .= will_bontrager_local_syndication_js_line($s);
				$post_result = will_bontrager_local_syndication_js_line($post_result);
			}
			else { $pre_result .= $s; }
		}
		if( $js )
		{ 
			$post_result .= '</script>';
			$response_content = will_bontrager_local_syndication_js_block($response_content);
		}
		return "$pre_result$response_content$post_result";
	} # function will_bontrager_local_syndication()

	function will_bontrager_local_syndication_js_line($s)
	{
		$s = str_replace("\\","\\"."\\",$s);
		$s = str_replace("'","\\"."'",$s);
		$s = str_replace('</',"<'+'/",$s);
		$s = str_replace('/>',"/'+'>",$s);
		$s = preg_replace('/\b(f)(orm)/i',"$1'+'$2",$s);
		$s = preg_replace('/\b(in)(put)/i',"$1'+'$2",$s);
		$s = preg_replace('/\b(scr)(ipt)/i',"$1'+'$2",$s);
		$s = preg_replace('/\b(ifr)(ame)/i',"$1'+'$2",$s);
		return "document.writeln('$s');\n";
	} # function will_bontrager_local_syndication_js_line()

	function will_bontrager_local_syndication_js_block(&$s)
	{
		$s2 = '';
		$s = str_replace("\r",'',$s);
		foreach( explode("\n",$s) as $line ) { $s2 .= will_bontrager_local_syndication_js_line($line); }
		return $s2;
	} # function will_bontrager_local_syndication_js_block()

	function will_bontrager_local_syndication_make_httphttps($s)
	{
		if( isset($_SERVER['HTTPS']) and ($_SERVER['HTTPS']=='on' or $_SERVER['HTTPS']=='1') ) { return 'https'; }
		return 'http';
	} # function will_bontrager_local_syndication_make_httphttps()

	add_shortcode('syndicate_local','will_bontrager_local_syndication');
} # if( ! function_exists('will_bontrager_local_syndication') )
?>
